--[[
	Title: Concommand
	
	Has some functions to make concommands and variables easier.
]]

--[[
	Function: commandToFnName
	
	This is an *internal function* for ULib to simplify making function names out of commands.
	_*DO NOT USE IN YOUR SCRIPTS*_

	Parameters:

		command - The command

	Returns:

		A function name to use for the command.
]]
function ULib.commandToFnName( command )
	local fn_name = "ccULX_" .. string.gsub( command, "%-", "M_" )
	fn_name = string.gsub( fn_name, "%+", "P_" )
	fn_name = string.gsub( fn_name, " ", "_" )
	
	return fn_name
end


--[[
	Function: CONCOMMAND
	
	This is a "beefed up" version of GMod's CONCOMMAND(). Instead of calling the function with just userid and args, it will call the function with userid, args, argv, argc.
	argv is a broken up version of args in a table, obeying quotation marks, argc is the number of values in argv.

	Parameters:
		
		command - The console command. IE, "ulx_kick".
		fn_call - The function to call when the command's called.
		cmd_help - *(Optional)* A help string for using the command.
		
	Returns:
	
		True on success, nil and error otherwise.
]]
function ULib.CONCOMMAND( command, fn_call, cmd_help )
	if not ULib.checkParams( { command, fn_call }, { "string", "function" } ) then
		return nil, "Incorrect parameters!"
	end

	local fn_name = ULib.commandToFnName( command )
	
	_G[ fn_name ] = function ( userid, args ) -- This function will handle the command callback
		local argv = ULib.splitArgs( args )
		local argc = table.getn( argv )

		fn_call( userid, args, argv, argc )
	end

	CONCOMMAND( command, _G[ fn_name ], cmd_help )
	
	return true
end


--[[
	Table: gSubCommands
	
	Used to support <BEGIN_SUBCONCOMMAND()> and <ADD_SUBCONCOMMAND()>
]]
local gSubCommands = {}


--[[
	Function: BEGIN_SUBCONCOMMAND
	
	This function will help you set up subset concommands. For example, instead of using "ulx_kick", "ulx_slay", you may want to use a more conventional format like "ulx kick" and "ulx slay".
	This function helps you acheive this. Call <BEGIN_SUBCONCOMMAND()> first to define the "catch all" function if they specify an unknown parameter of the command by itself ( IE, "ulx" using the examples above ).
	Then, call <ADD_SUBCONCOMMAND()> to add the subconcommands.

	Parameters:
		
		prefix - The console command prefix. IE, "ulx".
		fn_call - The default function to call.
		cmd_help - *(Optional)* A help string for using the command.
		
	Returns:
	
		True on success, nil and error otherwise.
]]
function ULib.BEGIN_SUBCONCOMMAND( prefix, fn_call, cmd_help )
	if not ULib.checkParams( { prefix, fn_call }, { "string", "function" } ) then
		return nil, "Incorrect parameters!"
	end

	local prefix_list = ULib.explode( prefix, "%s+" )
	local num = table.getn( prefix_list )
	local table = gSubCommands
	for i=1, num - 1 do -- Get to the table list we need
		table = gSubCommands[ prefix_list[ i ] ]
		if not table then
			return nil, "Recursive subcommands with prefixes that don't exist!"
		end
	end
	table[ prefix_list[ num ] ] = {}
	local table = table[ prefix_list[ num ] ] -- This will make it easier to handle
	

	local f = function ( userid, args, argv, argc ) -- This function will handle the command callback
		if type( table[ argv[ num ] ] ) == "function" then
			local fn = argv[ num ]
			for i=1, num do
				table.remove( argv, num - i + 1 ) -- We could use 1 here instead of taking at the end of what we need to, but this is more effecient. 
				args = string.gsub( args, "^%S+%s+(.*)$", "%1" )
			end
			table[ fn ]( userid, args, argv, argc - num )
		else
			fn_call( userid, args, argv, argc ) -- Fallback on default function.
		end
	end

	if num == 1 then
		return ULib.CONCOMMAND( prefix, f, cmd_help )
	else
		return ULib.ADD_SUBONCOMMAND( table.concat( prefix_list, " ", 1, num - 1 ), prefix_list[ num ], f, cmd_help )
	end
end


--[[
	Function: ADD_SUBCONCOMMAND
	
	See <BEGIN_SUBCONCOMMAND()>.

	Parameters:

		prefix - The console command prefix. IE, "ulx".
		command - The subconcommand. IE, "kick" if you want something like "ulx kick" for the final command.
		fn_call - The function to call when the command's called.
		cmd_help - *(Optional)* A help string for using the command.
		
	Returns:
	
		True on success, nil and error otherwise.
]]--
function ULib.ADD_SUBCONCOMMAND( prefix, command, fn_call, cmd_help )
	if not ULib.checkParams( { prefix, command, fn_call }, { "string", "string", "function" } ) then
		return nil, "Incorrect parameters!"
	end

	local prefix_list = ULib.explode( prefix, "%s+" )
	local num = table.getn( prefix_list )
	local table = gSubCommands
	for i=1, num do -- Get to the table list we need
		table = gSubCommands[ prefix_list[ i ] ]
		if not table then
			return nil, "Prefix does not exist! You must call BEGIN_SUBCONCOMMAND first."
		end
	end

	table[ command ] = fn_call
	
	return true
end


--[[
	Table: gCvars

	Used to support the cvar functions
]]
local gCvars = {}


--[[
	Function: CONVAR

	Adds a cvar which clients (with permission) can access and change.

	Parameters:

		cvar_name - The cvar name. IE, "ulx_hostname2".
		value - The value to start off at.
		cmd_access - The access for the command.
		cmd_help - *(Optional)* The help for the command.
		ucl - *(Optional, defaults to <mainUcl>)* The access list to use.

	Returns:

		A table of userids (empty if none found), nil and error message otherwise.
]]
function ULib.CONVAR( cvar_name, value, cmd_access, cmd_help, ucl )
	if not ULib.checkParams( { prefix, cvar_name, value }, { "string", "string", "string" } ) then return nil, "Incorrect parameters" end
	gCvars[ cvar_name ] = value
	ucl = ucl or mainUcl
	
	local f = function ( userid, args ) -- This function will handle the cvar ( command ) callback
		if not ucl:query( userid, cmd_access ) then
			ULib.console( userid, "You lack sufficient access to use this cvar!" )
			return
		end

		args = ULib.stripQuotes( args )		
		if args == "" then
			if cmd_help then
				ULib.console( userid, "\"" .. cvar_name .. "\" = \"" .. gCvars[ cvar_name ] .. "\"\n  " ) 
				ULib.console( userid, cmd_help .. "\n  CVAR generated by ULib" )
				return
			else
				ULib.console( userid, "\"" .. cvar_name .. "\" = \"" .. gCvars[ cvar_name ] .. "\"\n" ) 
				ULib.console( userid, "  CVAR generated by ULib" )
				return
			end
		else
			gCvars[ cvar_name ] = args
		end
	end
	
	CONCOMMAND( cvar_name, f, cmd_help )
end


--[[
	Function: ADD_SUBCONVAR

	Adds a sub cvar which clients (with permission) can access and change. See <ADD_SUBCONCOMMAND()>.

	Parameters:

		prefix - The prefix to use, IE: "ulx"
		cvar_name - The cvar name. IE, "hostname2".
		value - The value to start off at.
		cmd_access - The access for the command.
		cmd_help - *(Optional)* The help for the command.
		ucl - *(Optional, defaults to <mainUcl>)* The access list to use.

	Returns:

		A table of userids (empty if none found), nil and error message otherwise.
]]
function ULib.ADD_SUBCONVAR( prefix, cvar_name, value, cmd_access, cmd_help, ucl )
	if not ULib.checkParams( { prefix, cvar_name, value }, { "string", "string", "string" } ) then return nil, "Incorrect parameters" end
	gCvars[ prefix .. " " .. cvar_name ] = value
	ucl = ucl or mainUcl
	
	local f = function ( userid, args ) -- This function will handle the cvar ( command ) callback
		if not ucl:query( userid, cmd_access ) then
			ULib.console( userid, "You lack sufficient access to use this cvar!" )
			return
		end

		args = ULib.stripQuotes( args )		
		if args == "" then
			if cmd_help then
				ULib.console( userid, "\"" .. cvar_name .. "\" = \"" .. gCvars[ prefix .. " " .. cvar_name ] .. "\"\n  " )
				ULib.console( userid, cmd_help .. "\n  CVAR generated by ULib" )
				return
			else
				ULib.console( userid, "\"" .. cvar_name .. "\" = \"" .. gCvars[ prefix .. " " .. cvar_name ] .. "\"\n" )
				ULib.console( userid, "  CVAR generated by ULib" )
				return
			end
		else
			gCvars[ prefix .. " " .. cvar_name ] = args
		end
	end

	ULib.ADD_SUBCONCOMMAND( prefix, cvar_name, f, cmd_help )
end


--[[
	Function: getCvar

	Gets the string value from a cvar

	Parameters:

		cvar_name - The name of the cvar to retrieve
]]
function ULib.getCvar( cvar_name )
	return gCvars[ cvar_name ]
end