--[[
	Title: Player

	Has useful player-related functions.
]]


--[[
	Table: gSlapSounds

	These are the sounds used for slaps.
]]--
gSlapSounds = {
	"physics/body/body_medium_impact_hard1.wav",
	"physics/body/body_medium_impact_hard2.wav",
	"physics/body/body_medium_impact_hard3.wav",
	"physics/body/body_medium_impact_hard5.wav",
	"physics/body/body_medium_impact_hard6.wav",
	"physics/body/body_medium_impact_soft5.wav",
	"physics/body/body_medium_impact_soft6.wav",
	"physics/body/body_medium_impact_soft7.wav",
}


--[[
	Function: slap

	Slaps a targetid, can be a user or any entity.

	Parameters:

		target - Target id.
		damage - *(Optional, defaults to 0)* The amount of damage to inflict on the entity.
		power - *(Optional, defaults to 30)* The power of the slap.
		nosound - *(Optional, defaults to false)* If true, no sound will be played.
]]
function ULib.slap( target, damage, power, nosound )
	if _EntGetMoveType( target ) == MOVETYPE_OBSERVER then return end -- Nothing we can do.

	damage = damage or 0
	power = power or 30
	
	if ULib.isInVehicle( target ) then ULib.forceOutOfVehicle( target ) end

	if not nosound then -- Play a slap sound
		local sound_num = math.random( table.getn( gSlapSounds ) ) -- Choose at random
		ULib.play3DSound( gSlapSounds[ sound_num ], _EntGetPos( target ), 1.5 )
	end

	local direction = vector3( math.random( 20 )-10, math.random( 20 )-10, math.random( 20 )-5 ) -- Make it random, slightly biased to go up.
	ULib.applyAccel( target, power, direction )

	local newHp = _PlayerInfo( target, "health" ) - damage
	if newHp <= 0 then
		_PlayerKill( target )
		return
	end
	_PlayerSetHealth( target, newHp )
end


--[[
	Function: forceOutOfVehicle

	Forces a player out of a vehicle.

	Parameters:

		userid - The player to force out of a vehicle.
]]
function ULib.forceOutOfVehicle( userid )
	if not ULib.isInVehicle( userid ) then
		return nil, "Userid is not in a vehicle"
	end

	local vehicle = _EntGetParent( userid )
	local playerPos = _EntGetPos( userid )
	local playerPosNew = vecAdd( playerPos, vecMul( _EntGetRightVector( vehicle ), -80 ) )

	_PlayerGod( userid, false ) -- Let him be killed
	_PlayerKill( userid )
	_PlayerRespawn( userid )
	_EntSetPos( userid, playerPosNew )
end


--[[
	Function: isInVehicle

	Checks if the entity (player) is in a vehicle

	Parameters:

		userid - The userid to check
	
	Returns:
	
		True if they're in a vehicle, false otherwise.
]]
function ULib.isInVehicle( userid )
	if string.find( _EntGetType( _EntGetParent( userid ) ), "vehicle" ) then
		return true
	else
		return false
	end
end


--[[
	Table: gSteamIdToIp

	This will hold the IPs of connected steamids
]]--
local gSteamIdToIp = {}
local gNameToIp = {}


--[[
	Function: collectUserIp

	*THIS IS FOR INTERNAL USE*. This is what gives <gSteamIdToIp> it's values.
]]
function ULib.collectUserIp( name, ip, steamid )
	local beg = string.find( ip, ":" )
	ip = string.sub( ip, 1, beg - 1 )
	gNameToIp[ name ] = ip
end
HookEvent( "eventPlayerConnect", ULib.collectUserIp )


--[[
	Function: relateUserIp

	*THIS IS FOR INTERNAL USE*. This is what gives <gSteamIdToIp> it's values.
]]
function ULib.relateUserIp( name, userid, steamid )
	gSteamIdToIp[ steamid ] = gNameToIp[ name ]
	gNameToIp[ name ] = nil
end
HookEvent( "eventPlayerActive", ULib.relateUserIp )


--[[
	Function: getIp

	Parameters:

		userid - The user's id
		
	Returns:
	
		The user's IP
]]
function ULib.getIp( userid )
	return gSteamIdToIp[ _PlayerInfo( userid, "networkid" ) ]
end


--[[
	Function: getUsers

	Queries the UCL to see if a user has access.

	Parameters:

		identifier - A string of the identifier you'd like to match. IE, a partial player name.
		enable_keywords - *(Optional, defaults to false)* If true, keywords 
		"<ALL>", "<BLUE>", "<YELLOW>", "<GREEN>", and "<RED>" will be activated. They will return what you think they would (All players or teams).
		ignore_immunity - *(Optional, defaults to false)* If true, users with immunity will be returned.
		ucl - *(Optional, defaults to mainUcl)* The access list to use.

	Returns:

		A table of userids (empty if none found), nil and error message otherwise.
]]
function ULib.getUsers( identifier, ignore_immunity, enable_keywords, ucl )
	if not ULib.checkParam( identifier, "string" ) then
		return nil, "Invalid parameters!"
	end

	ucl = ucl or mainUcl

	local result = {}
	
	if enable_keywords then
		if string.upper( identifier ) == "<ALL>" then
			for i=1, _MaxPlayers() do
				if _PlayerInfo( i, "connected" ) then
					table.insert( result, i )
				end
			end
			return result
		elseif string.upper( identifier ) == "<BLUE>" then
			for i=1, _MaxPlayers() do
				if _PlayerInfo( i, "connected" ) and _PlayerInfo( i, "team" ) == TEAM_BLUE then
					table.insert( result, i )
				end
			end
			return result
		elseif string.upper( identifier ) == "<YELLOW>" then
			for i=1, _MaxPlayers() do
				if _PlayerInfo( i, "connected" ) and _PlayerInfo( i, "team" ) == TEAM_YELLOW then
					table.insert( result, i )
				end
			end
			return result
		elseif string.upper( identifier ) == "<GREEN>" then
			for i=1, _MaxPlayers() do
				if _PlayerInfo( i, "connected" ) and _PlayerInfo( i, "team" ) == TEAM_GREEN then
					table.insert( result, i )
				end
			end
			return result
		elseif string.upper( identifier ) == "<RED>" then
			for i=1, _MaxPlayers() do
				if _PlayerInfo( i, "connected" ) and _PlayerInfo( i, "team" ) == TEAM_RED then
					table.insert( result, i )
				end
			end
			return result
		end
	end

	local num = tonumber( identifier )
	if num and num > 0 and num <= _MaxPlayers() and _PlayerInfo( num, "connected" ) then -- Guess they mean a person's userid		
		if ignore_immunity or not ucl:query( num, ACCESS_IMMUNITY ) then -- If we're ignoring immunity or if they don't have immunity.
			return { num } -- And we're done!
		else
			return nil, "Passed a userid, but the userid had immunity"
		end
	end
	
	local user = string.lower( identifier ) -- We want to make this a case insensitive search

	for i=1, _MaxPlayers() do
		if _PlayerInfo( i, "connected" ) == true then
			temp = string.find( string.lower( _PlayerInfo( i, "name" ) ), user, 1, true ) -- No special characters.
			if temp then
				if ignore_immunity or not ucl:query( i, ACCESS_IMMUNITY ) then -- If we're ignoring immunity or if they don't have immunity.
					table.insert( result, i )
				end
			end
		end
	end

	return result
end


--[[
	Function: kick

	Kicks a user.

	Parameters:

		userid - The userid to kick.
		reason - The reason to give for kicking. *Note, providing a reason makes this a little unstable. If you want to kick bots or people on a LAN server, do NOT provide a reason.*
]]
function ULib.kick( userid, reason )
	if reason and type( reason ) == "string" then
		_ServerCommand( "kickid " .. _PlayerInfo( userid, "networkid" ) .. " \"" .. reason .. "\"\n" )
	else
		_ServerCommand( "kick " .. _PlayerInfo( userid, "name" ) .. "\n" )
	end
end


--[[
	Function: ban

	Bans a user.

	Parameters:

		userid - The userid to ban.
		time - The time in minutes to ban the person for, leave nil or 0 for permaban.
]]
function ULib.ban( userid, time )
	if not time or type( time ) ~= "number" then
		time = 0
	end

	_ServerCommand( "banid " .. time .. " " .. _PlayerInfo( target, "networkid" ) .. ";writeid\n" )
end


--[[
	Function: kickban

	Kicks and bans a user.

	Parameters:

		userid - The userid to kickban.
		time - The time in minutes to ban the person for, leave nil or 0 for permaban.
]]
function ULib.kickban( userid, time )
	if not time or type( time ) ~= "number" then
		time = 0
	end

	_ServerCommand( "banid " .. time .. " " .. _PlayerInfo( target, "networkid" ) .. " kick;writeid\n" )
end


--[[
	Function: getSteamLogin

	Retrieves a user's login name.

	Parameters:

		userid - The entity index location of the user.

	Returns:
	
		A lowercase string of the user's Steam login name. This is the username they use when logging into Steam.
]]
function ULib.getSteamLogin( userid )
	return _GetClientConVar_String( userid, "stmlgn" )
end