--[[
	Title: Utilities

	General GMod-related functions.
]]


--[[
	Function: execFile

	Executes a cfg file. The difference between this and "exec file" in console is that this function will run lua commands in the file, even on dedicated servers.
	It will treat "//" in lines as comments.

	Parameters:

		path - The location of the file to execute. Must be the full path from gmod9/ including the extension.

	Important:
	
		This function may not work for all lua commands, as it passes a userid of 0. ( IE: You couldn't use a command like "giveMeWeapon supahWep", because it wouldn't know who to give it to. )
]]
function ULib.execFile( file )
	if not _file.Exists( file ) then
		ULib.debug( "execFile was passed an invalid file!", ULib.ERR )
		return
	end

	local lines = ULib.readFile( file, "//" )

	for i=1, table.getn( lines ) do
		if gConsoleCommands[ command ] then -- This is a lua command!
			local command = trim( string.sub( lines[ i ], 1, string.find( lines[ i ], " " ) - 1 ) )
			local args = trim( string.sub( lines[ i ], string.find( lines[ i ], " " ) + 1 ) )

   			gConsoleCommands[ command ]( 0, args ) -- Note that a userid of 0 may cause problems with some functions
		else -- We're assuming it's just an average command to run
			_ServerCommand( lines[ i ] .. "\n" )
		end
	end
end


--[[
	Function: parseKeyValues

	Parses a keyvalue formatted string into a table.

	Parameters:

		str - The string to parse.

	Returns:

		The table, nil and error otherwise. *If you find you're missing information from the table, the file format might be incorrect.*
		
	Example format:
:test
:{
:	"howdy"   "bbq"
:
:	foo
:	{
:		"bar"   "false"
:	}
:
:}
]]
function ULib.parseKeyValues( str )
	if not ULib.checkParam( str, "string" ) then return nil, "Incorrect parameters!" end
	str = ULib.stripComments( str, "//" )
	
	local lines = ULib.explode( str, "\n" )
	lines = ULib.removeEmptyValues( lines ) -- Remove crap
	local pos = 1
	local t = {}

	while pos <= table.getn( lines ) do
		local args = ULib.splitArgs( lines[ pos ] )

		if table.getn( args ) == 1 then
			if pos ~= table.getn( lines ) and ULib.trim( lines[ pos + 1 ] ) == "{" then
				local endpos = pos + 2
				local inline = 0
				while true do -- We have breaks inside
					endpos = endpos + 1
					if endpos > table.getn( lines ) then return nil, "Bracket ( \"{\" and \"}\" ) mismatch!" end

					if ULib.trim( lines[ endpos ] ) == "{" then inline = inline + 1 end

					if ULib.trim( lines[ endpos ] ) == "}" then
						if inline > 0 then inline = inline - 1
						else break end
					end
				end

				t[ args[ 1 ] ] = ULib.parseKeyValues( table.concat( lines, "\n", pos + 2, endpos - 1 ) )

				pos = endpos
			else
				table.insert( t, args[ 1 ] )
			end
		elseif table.getn( args ) == 2 then
			t[ args[ 1 ] ] = args[ 2 ]
		else
			ULib.debug( "Incorrect syntax found while running parseKeyValues! Line was: \"" .. lines[ pos ] .. "\"", ULib.DEBUG )
			tprint( args )
		end
		pos = pos + 1
	end

	return t
end

--[[
	Function: makeKeyValues

	Makes a key values string from a table.

	Parameters:

		t - The table to make the keyvalues from. This can only contain tables, numbers, and strings.
		tab - *Only for internal use*, this helps make inline tables look better.
		completed - A list of table values that have already been parsed, this is *only for internal use* to make sure we don't hit an infinite loop.

	Returns:

		The string, nil and error otherwise.
		
	Notes:

		If you use numbers as keys in the table, just the values will be used.
		
	Example table format:
:{ test = { howdy = "bbq", foo = { bar = "false" } } }

	Example return format:
:test
:{
:	"howdy"   "bbq"
:
:	foo
:	{
:		"bar"   "false"
:	}
:
:}
]]
function ULib.makeKeyValues( t, tab, completed )
	if not ULib.checkParam( t, "table" ) then return nil, "Incorrect Parameters!" end

	tab = tab or ""
	completed = completed or {}
	if ULib.isInTable( completed, t ) then return "" end -- We've already done this table.
	table.insert( completed, t )
	str = ""

	for k, v in pairs( t ) do
		str = str .. tab
		if type( k ) ~= "number" then
			str = str .. ULib.serialize( k ) .. ULib.SPACE_METHOD
		end

		if type( v ) == "number" then v = tostring( v ) end
		if type( v ) == "table" then
			str = str .. ULib.NEWLINE .. tab .. "{" .. ULib.NEWLINE ..
				ULib.makeKeyValues( v, tab .. ULib.SPACE_METHOD, completed ) ..
				tab .. "}" .. ULib.NEWLINE

		else
			str = str .. ULib.serialize( v ) .. ULib.NEWLINE
		end
	end
	
	return str
end


--[[
	Function: play3DSound

	Plays a 3D sound, the further away from the point the player is, the softer the sound will be.

	Parameters:

		sound - The sound to play, relative to the sound folder.
		vector - The point to play the sound at
		volume - *(Optional, defaults to 1)* The volume to make the sound.
		pitch - *(Optional, defaults to 1)* The pitch to make the sound, 1 = normal
]]
function ULib.play3DSound( sound, vector, volume, pitch )
	volume = volume or 1
	pitch = pitch or 1

	local newent = _EntCreate( "info_null" ) -- info_null will be "deleted" by the engine as soon as it's spawned, but still let's us access it's point.
	_EntSetPos( newent, vector )
	_EntSpawn( newent )

	_EntEmitSoundEx( newent, sound, volume, pitch )
end